aboutsummaryrefslogtreecommitdiffhomepage
path: root/src/gpu
diff options
context:
space:
mode:
authorGravatar Jim Van Verth <jvanverth@google.com>2017-01-11 12:21:43 -0500
committerGravatar Skia Commit-Bot <skia-commit-bot@chromium.org>2017-01-11 18:03:35 +0000
commitd081ff314f07104adacaadc3d0f8f13dc741f016 (patch)
treef901365dd5ecfb32e284048fb8273a45d407fe1b /src/gpu
parentc456b73fef9589bbdc5eb83eaa83e53c357bb3da (diff)
More fixes for distance field paths
Disables use of SDFs for very small paths (because of blurring) and adds a border of 1 pixel in device space to handle antialiasing. BUG=chromium:677889 Change-Id: I81e49477c943d41523fd836e55abd696a985491f Reviewed-on: https://skia-review.googlesource.com/6832 Commit-Queue: Jim Van Verth <jvanverth@google.com> Reviewed-by: Brian Salomon <bsalomon@google.com>
Diffstat (limited to 'src/gpu')
-rw-r--r--src/gpu/ops/GrAADistanceFieldPathRenderer.cpp71
-rw-r--r--src/gpu/ops/GrAADistanceFieldPathRenderer.h5
2 files changed, 45 insertions, 31 deletions
diff --git a/src/gpu/ops/GrAADistanceFieldPathRenderer.h b/src/gpu/ops/GrAADistanceFieldPathRenderer.h
index 5d3480743d..202b114e23 100644
--- a/src/gpu/ops/GrAADistanceFieldPathRenderer.h
+++ b/src/gpu/ops/GrAADistanceFieldPathRenderer.h
@@ -71,8 +71,9 @@ private:
};
Key fKey;
GrDrawOpAtlas::AtlasID fID;
- SkRect fBounds;
- SkRect fTexCoords;
+ SkRect fBounds;
+ SkScalar fScale;
+ SkVector fTranslate;
SK_DECLARE_INTERNAL_LLIST_INTERFACE(ShapeData);
static inline const Key& GetKey(const ShapeData& data) {
diff --git a/src/gpu/ops/GrAADistanceFieldPathRenderer.cpp b/src/gpu/ops/GrAADistanceFieldPathRenderer.cpp
index c9b30f5154..455d76eb46 100644
--- a/src/gpu/ops/GrAADistanceFieldPathRenderer.cpp
+++ b/src/gpu/ops/GrAADistanceFieldPathRenderer.cpp
@@ -38,6 +38,7 @@ static int g_NumFreedShapes = 0;
#endif
// mip levels
+static const int kMinSize = 16;
static const int kSmallMIP = 32;
static const int kMediumMIP = 73;
static const int kLargeMIP = 162;
@@ -106,14 +107,17 @@ bool GrAADistanceFieldPathRenderer::onCanDrawPath(const CanDrawPathArgs& args) c
return false;
}
// only support paths with bounds within kMediumMIP by kMediumMIP,
// scaled to have bounds within 2.0f*kLargeMIP by 2.0f*kLargeMIP
// the goal is to accelerate rendering of lots of small paths that may be scaling
SkScalar maxScale = args.fViewMatrix->getMaxScale();
SkRect bounds = args.fShape->styledBounds();
SkScalar maxDim = SkMaxScalar(bounds.width(), bounds.height());
- return maxDim <= kMediumMIP && maxDim * maxScale <= 2.0f*kLargeMIP;
+ return maxDim >= kMinSize && maxDim <= kMediumMIP &&
+ maxDim * maxScale >= kMinSize && maxDim * maxScale <= 2.0f*kLargeMIP;
}
////////////////////////////////////////////////////////////////////////////////
@@ -398,29 +402,12 @@ private:
// set the bounds rect to the original bounds
shapeData->fBounds = bounds;
- // set up texture coordinates
- SkScalar texLeft = bounds.fLeft;
- SkScalar texTop = bounds.fTop;
- SkScalar texRight = bounds.fRight;
- SkScalar texBottom = bounds.fBottom;
-
- // transform original path's bounds to texture space
- texLeft *= scale;
- texTop *= scale;
- texRight *= scale;
- texBottom *= scale;
+ // set up path to texture coordinate transform
+ shapeData->fScale = scale;
dx -= SK_DistanceFieldPad + kAntiAliasPad;
dy -= SK_DistanceFieldPad + kAntiAliasPad;
- texLeft += atlasLocation.fX - dx;
- texTop += atlasLocation.fY - dy;
- texRight += atlasLocation.fX - dx;
- texBottom += atlasLocation.fY - dy;
-
- GrTexture* texture = atlas->getTexture();
- shapeData->fTexCoords.setLTRB(texLeft / texture->width(),
- texTop / texture->height(),
- texRight / texture->width(),
- texBottom / texture->height());
+ shapeData->fTranslate.fX = atlasLocation.fX - dx;
+ shapeData->fTranslate.fY = atlasLocation.fY - dy;
fShapeCache->add(shapeData);
fShapeList->addToTail(shapeData);
@@ -439,10 +426,15 @@ private:
const ShapeData* shapeData) const {
SkPoint* positions = reinterpret_cast<SkPoint*>(offset);
+ // outset bounds to include ~1 pixel of AA in device space
+ SkRect bounds = shapeData->fBounds;
+ SkScalar outset = SkScalarInvert(maxScale);
+ bounds.outset(outset, outset);
+
// vertex positions
// TODO make the vertex attributes a struct
- positions->setRectFan(shapeData->fBounds.left(), shapeData->fBounds.top(),
- shapeData->fBounds.right(), shapeData->fBounds.bottom(), vertexStride);
+ positions->setRectFan(bounds.left(), bounds.top(), bounds.right(), bounds.bottom(),
+ vertexStride);
// colors
for (int i = 0; i < kVerticesPerQuad; i++) {
@@ -450,11 +442,32 @@ private:
*colorPtr = color;
}
+ // set up texture coordinates
+ SkScalar texLeft = bounds.fLeft;
+ SkScalar texTop = bounds.fTop;
+ SkScalar texRight = bounds.fRight;
+ SkScalar texBottom = bounds.fBottom;
+
+ // transform original path's bounds to texture space
+ SkScalar scale = shapeData->fScale;
+ const SkVector& translate = shapeData->fTranslate;
+ texLeft *= scale;
+ texTop *= scale;
+ texRight *= scale;
+ texBottom *= scale;
+ texLeft += translate.fX;
+ texTop += translate.fY;
+ texRight += translate.fX;
+ texBottom += translate.fY;
+
// vertex texture coords
// TODO make these int16_t
SkPoint* textureCoords = (SkPoint*)(offset + sizeof(SkPoint) + sizeof(GrColor));
- textureCoords->setRectFan(shapeData->fTexCoords.left(), shapeData->fTexCoords.top(),
- shapeData->fTexCoords.right(), shapeData->fTexCoords.bottom(),
+ GrTexture* texture = atlas->getTexture();
+ textureCoords->setRectFan(texLeft / texture->width(),
+ texTop / texture->height(),
+ texRight / texture->width(),
+ texBottom / texture->height(),
vertexStride);
}
@@ -106,14 +107,17 @@ bool GrAADistanceFieldPathRenderer::onCanDrawPath(const CanDrawPathArgs& args) c
return false;
}
- // only support paths with bounds within kMediumMIP by kMediumMIP,
- // scaled to have bounds within 2.0f*kLargeMIP by 2.0f*kLargeMIP
- // the goal is to accelerate rendering of lots of small paths that may be scaling
+ // Only support paths with bounds within kMediumMIP by kMediumMIP,
+ // scaled to have bounds within 2.0f*kLargeMIP by 2.0f*kLargeMIP.
+ // For clarity, the original or scaled path should be at least kMinSize by kMinSize.
+ // TODO: revisit this last criteria with Joel's patch.
+ // The goal is to accelerate rendering of lots of small paths that may be scaling.
SkScalar maxScale = args.fViewMatrix->getMaxScale();
SkRect bounds = args.fShape->styledBounds();